/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 订单列表
 * @Date: 2024-01-16 12:00:38
 */
package model

import (
	"github.com/gogf/gf/v2/os/gtime"
	"github.com/gogf/gf/v2/util/gmeta"
	"github.com/tiger1103/gfast/v3/api/v1/common"
	comModel "github.com/tiger1103/gfast/v3/internal/app/common/model"
	memberModel "github.com/tiger1103/gfast/v3/internal/app/store/member/model"
)

// StoreOrderInfoRes is the golang structure for table store_order.
type StoreOrderInfoRes struct {
	gmeta.Meta       `orm:"table:store_order"`
	Uid              uint64                                 `orm:"uid" json:"uid" dc:"ucenterID"`             // ucenterID
	OrderId          uint64                                 `orm:"order_id" json:"orderId,string" dc:"订单编号"`  // 订单编号
	FreightPrice     float64                                `orm:"freight_price" json:"freightPrice" dc:"运费"` // 运费
	TotalAmount      float64                                `orm:"total_amount" json:"totalAmount" dc:"订单总价"` // 订单总价
	PayAmount        float64                                `orm:"pay_amount" json:"payAmount" dc:"支付金额"`     // 支付金额
	PayFreight       float64                                `orm:"pay_freight" json:"payFreight" dc:"支付运费"`   // 支付运费
	Paid             int                                    `orm:"paid" json:"paid" dc:"支付状态"`                // 支付状态
	PayTime          *gtime.Time                            `orm:"pay_time" json:"payTime" dc:"支付时间"`         // 支付时间
	PayType          string                                 `orm:"pay_type" json:"payType" dc:"支付类型"`         // 支付类型
	PayNo            string                                 `orm:"pay_no" json:"payNo" dc:"支付订单"`             // 支付订单
	OutTradeNo       uint64                                 `orm:"out_trade_no" json:"outTradeNo"   description:"交易订单号，发给第三方支付平台的"`
	CreatedAt        *gtime.Time                            `orm:"created_at" json:"createdAt" dc:"创建日期"`                                             // 创建日期
	UpdatedAt        *gtime.Time                            `orm:"updated_at" json:"updatedAt" dc:"更新日期"`                                             // 更新日期
	Status           int                                    `orm:"status" json:"status" dc:"订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)"` // 订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)
	Remark           string                                 `orm:"remark" json:"remark" dc:"备注"`                                                      // 备注
	BuyerMessage     string                                 `orm:"buyer_message" json:"buyerMessage"`                                                 // 买家留言
	AddressId        uint64                                 `orm:"address_id" json:"addressId"`                                                       // 收货地址
	BonusPoints      uint                                   `orm:"bonus_points" json:"bonusPoints"  description:"赠送积分"`
	OrderType        int                                    `orm:"order_type" json:"orderType" dc:"订单类型"`                // 订单类型
	VerifyCode       string                                 `orm:"verify_code" json:"verifyCode" dc:"核销码"`               // 核销码
	DiscountAmount   float64                                `json:"discountAmount" dc:"优惠金额,计算 totalAmount - payAmount"` // 优惠金额
	OrderGoodsDetail []*LinkedStoreOrderDetailInfoRes       `orm:"with:order_id=order_id" json:"orderGoodsDetail"`
	MemberInfo       *memberModel.LinkedStoreMemberInfoRes  `orm:"with:uid=uid" json:"memberInfo"`
	AddressInfo      *memberModel.StoreMemberAddressInfoRes `orm:"with:id=address_id" json:"addressInfo"` //地址信息
}

// StoreOrderListRes 订单列表返回结果
type StoreOrderListRes struct {
	Uid              uint64                                `json:"uid" dc:"ucenterID"`
	OrderId          uint64                                `json:"orderId,string" dc:"订单编号"`
	FreightPrice     float64                               `json:"freightPrice" dc:"运费"`
	TotalAmount      float64                               `json:"totalAmount" dc:"订单总价"`
	PayAmount        float64                               `json:"payAmount" dc:"支付金额"`
	PayFreight       float64                               `json:"payFreight" dc:"支付运费"`
	Paid             int                                   `json:"paid" dc:"支付状态"`
	PayTime          *gtime.Time                           `json:"payTime" dc:"支付时间"`
	PayType          string                                `json:"payType" dc:"支付类型"`
	PayNo            string                                `json:"payNo" dc:"支付订单"`
	OutTradeNo       uint64                                `json:"outTradeNo"   description:"交易订单号，发给第三方支付平台的"`
	CreatedAt        *gtime.Time                           `json:"createdAt" dc:"创建日期"`
	Status           int                                   `json:"status" dc:"订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)"`
	Remark           string                                `json:"remark" dc:"备注"`
	BuyerMessage     string                                `json:"buyerMessage" dc:"用户留言"`
	AddressId        uint64                                `json:"addressId"`
	OrderGoodsDetail []*LinkedStoreOrderDetailInfoRes      `orm:"with:order_id=order_id" json:"orderGoodsDetail"`
	MemberInfo       *memberModel.LinkedStoreMemberInfoRes `orm:"with:uid=uid" json:"memberInfo"`
	BonusPoints      uint                                  `json:"bonusPoints"  description:"赠送积分"`
	OrderType        int                                   `json:"orderType" dc:"订单类型"`
	VerifyCode       string                                `json:"verifyCode" dc:"核销码"`
}

// StoreOrderSearchReq 分页请求参数
type StoreOrderSearchReq struct {
	comModel.PageReq
	KeywordsType int    `p:"keywordsType" dc:"关键词类型"`
	Keywords     string `p:"keywords" dc:"关键词"`
	Status       string `p:"status" dc:"订单状态"`
	Uid          string `p:"uid" dc:"订单用户"`
}

// StoreOrderSearchRes 列表返回结果
type StoreOrderSearchRes struct {
	comModel.ListRes
	List []*StoreOrderListRes `json:"list"`
}

// StoreOrderAddReq 添加操作请求参数
type StoreOrderAddReq struct {
	Uid          uint64      `p:"uid"  dc:"ucenterID"`
	OrderId      uint64      `p:"orderId"  dc:"订单编号"`
	FreightPrice float64     `p:"freightPrice"  dc:"运费"`
	TotalAmount  float64     `p:"totalAmount"  dc:"订单总价"`
	PayAmount    float64     `p:"payAmount"  dc:"支付金额"`
	PayFreight   float64     `p:"payFreight"  dc:"支付运费"`
	Paid         int         `p:"paid"  dc:"支付状态"`
	PayTime      *gtime.Time `p:"payTime"  dc:"支付时间"`
	PayType      string      `p:"payType"  dc:"支付类型"`
	PayNo        string      `p:"payNo"  dc:"支付订单"`
	Status       int         `p:"status"  dc:"订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)"`
	Remark       string      `p:"remark"  dc:"备注"`
	BuyerMessage string      `p:"buyerMessage"  dc:"买家留言"`
	AddressId    uint64      `p:"addressId" dc:"收货地址ID"`
}

// StoreOrderEditReq 修改操作请求参数
type StoreOrderEditReq struct {
	OrderId     uint64  `p:"orderId"  dc:"订单编号"`
	PayAmount   float64 `p:"payAmount"  dc:"应支付金额"`
	PayFreight  float64 `p:"payFreight"  dc:"应支付运费"`
	BonusPoints uint    `p:"bonusPoints" dc:"赠送积分"`
	Remark      string  `p:"remark"  dc:"备注"`
}

// StoreOrderCreateReq 创建订单请求
type StoreOrderCreateReq struct {
	AddressId    uint64                          `p:"addressId" v:"required#请选择收货地址"`
	GoodsList    []*LinkedStoreOrderDetailAddReq `p:"goodsList" dc:"订单商品明细列表"`
	Coupons      []string                        `p:"coupons" dc:"优惠券列表"`
	Points       uint                            `p:"points" dc:"使用积分购物"`
	Remark       string                          `p:"remark" dc:"订单备注"`
	ShippingType int                             `p:"shippingType" dc:"配送方式"`
	BuyerMessage string                          `p:"buyerMessage" dc:"买家留言"`
	OrderType    int                             `p:"orderType" dc:"订单类型"`
}

// StoreOrderDetailDeliveryReq 订单发货请求参数
type StoreOrderDetailDeliveryReq struct {
	OrderId        uint64                    `p:"orderId"`        // 订单号
	DeliveryType   int                       `p:"deliveryType"`   // 发货方式
	ExpressCompany uint64                    `p:"expressCompany"` // 快递公司
	ExpressNo      string                    `p:"expressNo"`      // 快递单号
	Deliveryman    uint64                    `p:"deliveryman"`    // 送货人
	Remark         string                    `p:"remark"`         // 备注
	IsSplitDeliver bool                      `p:"isSplitDeliver"` // 是否拆分发货
	DeliveryList   []*StoreOrderDeliveryList `p:"deliveryList"`   // 拆分发货列表
}

// StoreOrderDeliveryList 订单发货列表
type StoreOrderDeliveryList struct {
	OrderId           uint64 `p:"orderId"`
	Id                uint64 `p:"id"`
	CurrentDeliverNum uint   `id:"currentDeliverNum"`
}

// StoreOrderVerifyReq 核销订单请求参数
type StoreOrderVerifyReq struct {
	OrderId    uint64 `p:"orderId" v:"required#请输入订单号"`
	VerifyCode string `p:"verifyCode" v:"required#请输入核销码"`
}

// StoreOrderPayArgs 订单支付参数
type StoreOrderPayArgs struct {
	Uid           uint64
	OrderId       uint64
	PayType       string
	Openid        string // 微信支付jssdk需要openid
	WechatPayType int    //微信支付方式   0 =jssdk 支付  0=1小程序支付
}

// StoreOrderStatReq 订单统计请求参数
type StoreOrderStatReq struct {
	StartDate string `p:"startDate" dc:"开始日期"`
	EndDate   string `p:"endDate" dc:"结束日期"`
}

// StoreOrderStatRes 订单统计返回结果
type StoreOrderStatRes struct {
	Year  int `json:"year"`
	Month int `json:"month"`
	Day   int `json:"day"`
	Num   int `json:"num"`
}

// StoreOrderMemberSearchReq 用户订单列表请求参数
type StoreOrderMemberSearchReq struct {
	common.PageReq
	Uid         uint64 `dc:"用户ID，不需要提交，自动获取"`
	GoodsName   string `p:"goodsName" dc:"商品名称"`
	Status      string `p:"status" dc:"订单状态"`
	PayStatus   string `p:"payStatus" dc:"支付状态"`
	AfterSale   bool   `p:"afterSale" dc:"售后状态"`
	WaitComment bool   `p:"waitComment" dc:"待评价"`
}

// StoreOrderMemberOrderListRes 用户订单列表返回结果
type StoreOrderMemberOrderListRes struct {
	Uid              uint64                           `dc:"用户ID，不需要提交，自动获取"`
	OrderId          uint64                           `orm:"order_id" json:"orderId,string" dc:"订单编号"`                                          // 订单编号
	FreightPrice     float64                          `orm:"freight_price" json:"freightPrice" dc:"运费"`                                         // 运费
	TotalAmount      float64                          `orm:"total_amount" json:"totalAmount" dc:"订单总价"`                                         // 订单总价
	PayAmount        float64                          `orm:"pay_amount" json:"payAmount" dc:"支付金额"`                                             // 支付金额
	PayFreight       float64                          `orm:"pay_freight" json:"payFreight" dc:"支付运费"`                                           // 支付运费
	Status           int                              `orm:"status" json:"status" dc:"订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)"` // 订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)
	OrderType        int                              `orm:"order_type" json:"orderType" dc:"订单类型"`                                             // 订单类型
	DiscountAmount   float64                          `json:"discountAmount" dc:"优惠金额,计算 totalAmount - payAmount"`                              // 优惠金额
	OrderGoodsDetail []*LinkedStoreOrderDetailInfoRes `orm:"with:order_id=order_id" json:"orderGoodsDetail"`
	CreatedAt        *gtime.Time                      `json:"createdAt" dc:"创建日期"`
	PayTime          *gtime.Time                      `json:"payTime" dc:"支付时间"`
}

// StoreOrderMemberSearchRes 列表返回结果
type StoreOrderMemberSearchRes struct {
	comModel.ListRes
	List []*StoreOrderMemberOrderListRes `json:"list"`
}

// StoreOrderMemberOrderInfoRes 用户订单详情返回结果
type StoreOrderMemberOrderInfoRes struct {
	OrderId          uint64                                 `orm:"order_id" json:"orderId,string" dc:"订单编号"`  // 订单编号
	FreightPrice     float64                                `orm:"freight_price" json:"freightPrice" dc:"运费"` // 运费
	TotalAmount      float64                                `orm:"total_amount" json:"totalAmount" dc:"订单总价"` // 订单总价
	PayAmount        float64                                `orm:"pay_amount" json:"payAmount" dc:"支付金额"`     // 支付金额
	PayFreight       float64                                `orm:"pay_freight" json:"payFreight" dc:"支付运费"`   // 支付运费
	Paid             int                                    `orm:"paid" json:"paid" dc:"支付状态"`                // 支付状态
	PayTime          *gtime.Time                            `orm:"pay_time" json:"payTime" dc:"支付时间"`         // 支付时间
	PayType          string                                 `orm:"pay_type" json:"payType" dc:"支付类型"`         // 支付类型
	OutTradeNo       string                                 `orm:"out_trade_no" json:"outTradeNo"   description:"交易订单号，发给第三方支付平台的"`
	CreatedAt        *gtime.Time                            `orm:"created_at" json:"createdAt" dc:"创建日期"`                                             // 创建日期
	Status           int                                    `orm:"status" json:"status" dc:"订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)"` // 订单状态(-1:已取消;0:待支付;1:待发货;2:待收货;3:已收货;4:待评价;5:申请退款;6:已退款)
	Remark           string                                 `orm:"remark" json:"remark" dc:"备注"`                                                      // 备注
	BuyerMessage     string                                 `orm:"buyer_message" json:"buyerMessage"`                                                 // 买家留言
	AddressId        uint64                                 `orm:"address_id" json:"addressId"`                                                       // 收货地址
	BonusPoints      uint                                   `orm:"bonus_points" json:"bonusPoints"  description:"赠送积分"`
	OrderType        int                                    `orm:"order_type" json:"orderType" dc:"订单类型"`                // 订单类型
	DiscountAmount   float64                                `json:"discountAmount" dc:"优惠金额,计算 totalAmount - payAmount"` // 优惠金额
	OrderGoodsDetail []*LinkedStoreOrderDetailInfoRes       `orm:"with:order_id=order_id" json:"orderGoodsDetail"`
	AddressInfo      *memberModel.StoreMemberAddressInfoRes `orm:"with:id=address_id" json:"addressInfo"` //地址信息
}

// StoreOrderConfirmReq 确认订单请求参数
type StoreOrderConfirmReq struct {
	AddressId uint64                          `p:"addressId" v:"required#请选择收货地址"`
	GoodsList []*LinkedStoreOrderDetailAddReq `p:"goodsList" dc:"订单商品明细列表"`
	Coupons   []string                        `p:"coupons" dc:"优惠券列表"`
	Points    uint                            `p:"points" dc:"使用积分购物"`
}

type StoreOrderConfirmRes struct {
	FreightAmount float64 `json:"freightAmount"` // 运费金额
	TotalAmount   float64 `json:"totalAmount"`   // 订单总价
	PayAmount     float64 `json:"payAmount"`     // 支付金额
	PayFreight    float64 `json:"payFreight"`    // 支付运费
	CouponAmount  float64 `json:"couponAmount"`  // 优惠金额
	PointsAmount  float64 `json:"pointsAmount"`  // 积分抵扣金额
	UsedPoints    uint    `json:"usedPoints"`    // 已使用积分
}
