/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 订单支付
 * @Date: 2024/3/14 10:56
 */

package service

import (
	"context"
	model2 "github.com/tiger1103/gfast/v3/internal/app/store/finance/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/order/model"
)

type IStorePayment interface {
	// WechatPay 微信支付
	WechatPay(ctx context.Context, openid string, wechatPayType int, orderInfo *model.StoreOrderInfoRes) (jssdkConfig interface{}, err error)
	// WechatNativePay 微信扫码支付，返回的是二唯码url
	WechatNativePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (codeUrl string, err error)
	// Alipay 支付宝支付
	Alipay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error)
	// BalancePay 余额支付
	BalancePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error)
	// OfflinePay 线下支付
	OfflinePay(ctx context.Context, orderInfo *model.StoreOrderInfoRes) (err error)
	// WechatFriendPay 微信好友支付
	WechatFriendPay(ctx context.Context, openid string, orderInfo *model.StoreOrderInfoRes) (jssdkConfig interface{}, err error)
	//用户余额充值 微信支付
	WeChatRecharge(ctx context.Context, openid string, wechatPayType int, rechargeInfo *model2.StoreFinanceRechargeLogInfoRes) (jssdkConfig interface{}, err error)
}

var localStorePayment IStorePayment

func StorePayment() IStorePayment {
	if localStorePayment == nil {
		panic("implement not found for interface IStorePayment, forgot register?")
	}
	return localStorePayment
}

func RegisterStorePayment(i IStorePayment) {
	localStorePayment = i
}
