/**
 * @Company: 云南奇讯科技有限公司
 * @Author: yxf
 * @Description: 微信支付
 * @Date: 2024/8/12 下午6:02
 */

package service

import (
	"context"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/payment/reverse/response"
	"github.com/tiger1103/gfast/v3/api/v1/store/wechat"
	"github.com/tiger1103/gfast/v3/internal/app/store/wechat/model"
	"net/http"
)

type IWechatPay interface {

	//小程序支付创建预支付订单
	AppletMiniPay(ctx context.Context, params *model.WechatPayPayParams) (res *model.WechatPayPrepayConfig, err error)
	//
	// Prepay
	//  @Description: 创建预支付订单
	//  @param ctx
	//  @param params
	//  @return jssdkConfig
	//  @return err
	//
	Prepay(ctx context.Context, params *model.WechatPayPayParams) (res *model.WechatPayPrepayConfig, err error)
	//
	// Native
	//  @Description: 扫码支付
	//  @param ctx
	//  @param params
	//  @return codeUrl
	//  @return err
	//
	Native(ctx context.Context, params *model.WechatPayPayParams) (codeUrl string, err error)
	//
	// QueryOrder
	//  @Description: 查询支订单
	//  @param ctx
	//  @return err
	//
	QueryOrder(ctx context.Context, outTradeNo string) (res *wechat.WechatPayQuerOrderRes, err error)
	//
	// Notify
	//  @Description: 处理支付结果通知
	//  @param ctx
	//  @return err
	//
	Notify(ctx context.Context, r *http.Request) (err error)
	//
	// Refunds
	//  @Description: 处理订单退款
	//  @param ctx
	//  @return err
	//
	Refunds(ctx context.Context) (err error)
	//
	// QueryRefund
	//  @Description: 查询退款订单
	//  @param ctx
	//  @return err
	//
	QueryRefund(ctx context.Context) (err error)
	//
	// Close
	//  @Description: 关闭订单
	//  @param ctx
	//  @return err
	//
	Close(ctx context.Context, outTradeNo string) (res *response.ResponseReserve, err error)
}

var localWechatPay IWechatPay

func WechatPay() IWechatPay {
	if localWechatPay == nil {
		panic("implement not found for interface IWechatPay, forgot register?")
	}
	return localWechatPay
}

func RegisterWechatPay(i IWechatPay) {
	localWechatPay = i
}
