/*
* @desc:用户充值记录
* @company:云南奇讯科技有限公司
* @Author: whs
* @Date:   2024/5/28 9:35
 */

package controller

import (
	"context"
	"errors"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/tiger1103/gfast/v3/api/v1/storeApp/finance"
	commonService "github.com/tiger1103/gfast/v3/internal/app/common/service"
	storeConsts "github.com/tiger1103/gfast/v3/internal/app/store/consts/finance"
	"github.com/tiger1103/gfast/v3/internal/app/store/finance/model"
	"github.com/tiger1103/gfast/v3/internal/app/store/finance/service"
	storeCommonModel "github.com/tiger1103/gfast/v3/internal/app/store/storeCommon/model"
	ucenterService "github.com/tiger1103/gfast/v3/internal/app/ucenter/service"
	"github.com/tiger1103/gfast/v3/library/liberr"
	"strconv"
)

type storeAppFinanceRechargeLogController struct {
}

var StoreAppFinanceRechargeLog = new(storeAppFinanceRechargeLogController)

// 创建充值订单
func (c *storeAppFinanceRechargeLogController) Add(ctx context.Context, req *finance.StoreAppFinanceRechargeLogAddReq) (res *finance.StoreAppFinanceRechargeLogAddRes, err error) {
	memberId := ucenterService.Context().GetMemberId(ctx)
	req.StoreFinanceRechargeLogAddReq.Uid = memberId
	// 生成订单ID

	res = new(finance.StoreAppFinanceRechargeLogAddRes)

	RechargeNo, err := commonService.SnowID().GenID() //充值单号
	req.StoreFinanceRechargeLogAddReq.RechargeNo = strconv.FormatUint(RechargeNo, 10)
	//充值赠送金额配置
	var rechargeConfig = new(model.StoreFinanceRechargeInfoRes)
	rechargeConfig, err = service.StoreFinanceRecharge().GetByAmount(ctx, req.RechargeAmount)
	if rechargeConfig != nil {
		req.GiftAmount = rechargeConfig.GiftAmount
		req.RechargeAmount = rechargeConfig.RechargeAmount
	}
	err = service.StoreFinanceRechargeLog().Add(ctx, req.StoreFinanceRechargeLogAddReq)
	if err == nil {
		res.RechargeNo = strconv.FormatUint(RechargeNo, 10)
	}
	return
}

func (c *storeAppFinanceRechargeLogController) GetByRechargeNo(ctx context.Context, req *finance.StoreAppFinanceRechargeLogGetReq) (res *finance.StoreFinanceRechargeLogGetRes, err error) {
	res = new(finance.StoreFinanceRechargeLogGetRes)
	res.StoreFinanceRechargeLogInfoRes, err = service.StoreFinanceRechargeLog().GetByRechargeNo(ctx, req.RechargeNo)
	return
}

func (c *storeAppFinanceRechargeLogController) BalancePay(ctx context.Context, req *finance.StoreAppFinanceBalanceRechargeReq) (res *finance.StoreAppFinanceBalanceRechargeRes, err error) {
	err = g.Try(ctx, func(ctx context.Context) {
		memberInfo := ucenterService.Context().GetLoginMember(ctx)
		wechatPayType := 0 ////jssdk支付
		if req.Openid == "" {
			req.Openid = memberInfo.WxOpenid
			wechatPayType = 1 //小程序支付
		}

		rechargeInfo := new(finance.StoreFinanceRechargeLogGetRes)
		rechargeInfo.StoreFinanceRechargeLogInfoRes, err = service.StoreFinanceRechargeLog().GetByRechargeNo(ctx, req.RechargeNo)
		if err != nil || rechargeInfo.StoreFinanceRechargeLogInfoRes == nil {
			liberr.ErrIsNil(ctx, errors.New("充值订单不存在"))
		}

		if rechargeInfo.StoreFinanceRechargeLogInfoRes.Uid != memberInfo.Id {
			liberr.ErrIsNil(ctx, errors.New("没有操作权限"))
		}

		if rechargeInfo.StoreFinanceRechargeLogInfoRes.Paid == storeConsts.BALANCE_RECHARGE_PAY_YES {
			liberr.ErrIsNil(ctx, errors.New("订单已支付"))
		}

		res = &finance.StoreAppFinanceBalanceRechargeRes{PayType: req.PayType}
		res.Result, err = service.StoreFinanceRechargeLog().Pay(ctx, &model.StoreFinanceRechargePayArgs{
			Uid:           ucenterService.Context().GetMemberId(ctx),
			RechargeNo:    req.RechargeNo,
			PayType:       req.PayType,
			Openid:        req.Openid,
			WechatPayType: wechatPayType,
		})
		liberr.ErrIsNil(ctx, err)
	})
	return
}

func (c *storeAppFinanceRechargeLogController) TextPay(ctx context.Context, req *finance.StoreFinanceBalanceRechargeTextReq) (res *finance.StoreFinanceRechargeLogGetRes, err error) {

	eventData := &storeCommonModel.StorePaymentEventData{
		OutTradeNo:    "762821826257616897",
		Message:       "",
		OpenId:        "oHQRAwTT9L77oSoLyD9K3QrZdQgY",
		PayType:       "wechat_pay",
		TransactionId: "",
		Attach:        "balance",
		Paid:          1,
	}

	err = service.StoreFinanceRechargeLog().HandlePayStatus(ctx, eventData)
	liberr.ErrIsNil(ctx, err, "订单支付状态变更事件处理失败")
	return
}
