/*
* @desc:中间件
* @company:云南奇讯科技有限公司
* @Author: yixiaohu<yxh669@qq.com>
* @Date:   2022/9/23 15:05
 */

package middleware

import (
	"github.com/gogf/gf/v2/encoding/gurl"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/net/ghttp"
	"github.com/gogf/gf/v2/text/gregex"
	"github.com/gogf/gf/v2/util/gconv"
	"github.com/tiger1103/gfast/v3/internal/app/ucenter/model"
	"github.com/tiger1103/gfast/v3/internal/app/ucenter/service"
	"github.com/tiger1103/gfast/v3/library/libResponse"
)

func init() {
	service.RegisterMiddleware(New())
}

func New() *sMiddleware {
	return &sMiddleware{}
}

type sMiddleware struct{}

// Ctx 自定义上下文对象
func (s *sMiddleware) Ctx(r *ghttp.Request) {
	var err error
	ctx := r.GetCtx()
	r.GetSessionId()
	token, err := gregex.ReplaceString(`Bearer\s+`, "", r.Header.Get("Authorization"))
	if token == "" {
		token = r.Session.MustGet("member_token").String()
		if token == "" {
			token, _ = gurl.Decode(r.Cookie.Get("member_token", "").String())
		}
		r.Header.Set("Authorization", "Bearer "+token)
	}

	// 初始化登录用户信息
	data, err := service.GfToken().ParseToken(r)
	if err != nil || data == nil {
		r.Session.Remove("memberId")
		r.Session.Remove("memberUuid")
		r.Session.Remove("memberInfo")
		// 执行下一步请求逻辑
		r.Middleware.Next()
	} else {
		context := new(model.Context)
		err = gconv.Struct(data.Data, &context.Member)
		// 重新获取用户信息
		newMemberInfo, err2 := service.UCenterMembers().GetById(r.GetCtx(), context.Member.Id)
		if err2 != nil || newMemberInfo == nil {
			g.Log().Error(ctx, err)
		} else {
			context.Member = newMemberInfo
		}
		if err != nil {
			g.Log().Error(ctx, err)
			// 执行下一步请求逻辑
			r.Middleware.Next()
		}
		service.Context().Init(r, context)
		r.Session.Set("memberId", context.Member.Id)
		r.Session.Set("memberUuid", context.Member.Uuid)
		r.Session.Set("memberInfo", context.Member)
	}
	// 执行下一步请求逻辑
	r.Middleware.Next()
}

// Auth 权限验证中间件
func (s *sMiddleware) Auth(r *ghttp.Request) {
	member := service.Context().GetLoginMember(r.GetCtx())
	if member == nil {
		libResponse.FailJson(true, r, "请先登录！")
	}
	// 执行下一步请求逻辑
	r.Middleware.Next()
}
