/*
* @desc:用户登录
* @company:云南奇讯科技有限公司
* @Author: yixiaohu<yxh669@qq.com>
* @Date:   2023/2/24 11:39
 */

package controller

import (
	"context"
	"fmt"
	"github.com/ArtisanCloud/PowerWeChat/v3/src/miniProgram/auth/response"
	resp "github.com/ArtisanCloud/PowerWeChat/v3/src/miniProgram/phoneNumber/response"
	"github.com/gogf/gf/v2/crypto/gmd5"
	"github.com/gogf/gf/v2/encoding/gurl"
	"github.com/gogf/gf/v2/errors/gerror"
	"github.com/gogf/gf/v2/frame/g"
	"github.com/gogf/gf/v2/util/gconv"
	"github.com/tiger1103/gfast/v3/api/v1/ucenter/admin"
	"github.com/tiger1103/gfast/v3/api/v1/wechat"
	"github.com/tiger1103/gfast/v3/internal/app/ucenter/model"
	"github.com/tiger1103/gfast/v3/internal/app/ucenter/model/do"
	service2 "github.com/tiger1103/gfast/v3/internal/app/ucenter/service"
	"github.com/tiger1103/gfast/v3/internal/app/wechat/service"
	"github.com/tiger1103/gfast/v3/library/libUtils"
)

var Login = new(loginController)

type loginController struct {
	BaseController
}

func (c *loginController) WxUrl(ctx context.Context, req *wechat.WxUrlReq) (res *wechat.WxUrlRes, err error) {
	res = new(wechat.WxUrlRes)
	res.Url, err = service.Wechat().GetWxUrl(ctx)
	return
}

/*** 小程序登录***********/
func (c *loginController) WxLogin(ctx context.Context, req *wechat.WxLoginReq) (res *wechat.WxLoginRes, err error) {
	var (
		data       *response.ResponseCode2Session
		phoneData  *resp.ResponseGetUserPhoneNumber
		token      string
		doMember   *do.UCenterMembers
		memberInfo *model.UCenterMembersInfoRes
	)
	data, err = service.Wechat().Session(ctx, req.Code)
	if err != nil {
		return
	}
	if data.ResponseMiniProgram.ErrCode != 0 {
		err = gerror.New(fmt.Sprintf("errCode: %d , errMsg: %s", data.ResponseMiniProgram.ErrCode, data.ResponseMiniProgram.ErrMsg))
		return
	}
	phoneData, err = service.Wechat().GetUserPhoneNumber(ctx, req.PhoneCode)
	if err != nil {
		return
	}
	if phoneData.ErrCode != 0 {
		err = gerror.New(fmt.Sprintf("errCode: %d , errMsg: %s", phoneData.ErrCode, phoneData.ErrMsg))
		return
	}

	r := g.RequestFromCtx(ctx)
	ip := libUtils.GetClientIp(ctx)
	userAgent := libUtils.GetUserAgent(ctx)
	//token, err = c.doLogin(ctx, phoneData.PhoneInfo.PhoneNumber, data.OpenID)
	doMember, err = service2.UCenterMembers().GetUserByMobile(ctx, phoneData.PhoneInfo.PhoneNumber)
	if err != nil {
		// 保存登录失败的日志信息
		service2.UCenterLoginLog().Add(ctx, &admin.UCenterLoginLogParams{
			Status:    0,
			Username:  phoneData.PhoneInfo.PhoneNumber,
			Ip:        ip,
			UserAgent: userAgent,
			Msg:       "账号不存在",
			Module:    "小程序登录",
		})
		err = gerror.New("账号不存在，请先注册账号")
		return
	}
	err = service2.UCenterMembers().UpdateLoginInfo(ctx, gconv.Uint64(doMember.Id), ip, data.OpenID)
	if err != nil {
		return
	}
	// 报存登录成功的日志信息
	service2.UCenterLoginLog().Add(ctx, &admin.UCenterLoginLogParams{
		Status:    1,
		Username:  gconv.String(doMember.Username),
		Ip:        ip,
		UserAgent: userAgent,
		Msg:       "登录成功",
		Module:    "小程序登录",
	})

	key := gconv.String(doMember.Id) + "-" + gmd5.MustEncryptString(gconv.String(doMember.Username)) + gmd5.MustEncryptString(gconv.String(doMember.Password))
	if g.Cfg().MustGet(ctx, "gfToken.multiLogin").Bool() {
		key = gconv.String(doMember.Id) + "-" + gmd5.MustEncryptString(gconv.String(doMember.Username)) + gmd5.MustEncryptString(gconv.String(doMember.Password)+ip+userAgent)
	}
	memberInfo, err = service2.UCenterMembers().GetById(ctx, gconv.Uint64(doMember.Id))
	token, err = service2.GfToken().GenerateToken(ctx, key, memberInfo)
	if err != nil {
		g.Log().Error(ctx, err)
		err = gerror.New("登录失败，" + err.Error())
		return
	}
	r.Cookie.SetCookie("member_token", gurl.Encode(token), r.Request.Host, "/", 0)
	res = &wechat.WxLoginRes{
		Token: token,
	}
	return
}

// 微信公众号登录
func (c *loginController) WeChatOfficialLogin(ctx context.Context, req *wechat.WxLogin2Req) (res *wechat.WxLogin2Res, err error) {
	var (
		doMember   *do.UCenterMembers
		token      string
		memberInfo *model.UCenterMembersInfoRes
	)
	r := g.RequestFromCtx(ctx)
	ip := libUtils.GetClientIp(ctx)
	userAgent := libUtils.GetUserAgent(ctx)
	doMember, err = service2.UCenterMembers().GetUserByOpenID(ctx, req.OpenId)
	if err != nil {
		err = gerror.New("用户信息不存在,请先注册账号绑定微信")
		return
	}
	err = service2.UCenterMembers().UpdateLoginInfo(ctx, gconv.Uint64(doMember.Id), ip)
	if err != nil {
		return
	}

	// 报存登录成功的日志信息
	service2.UCenterLoginLog().Add(ctx, &admin.UCenterLoginLogParams{
		Status:    1,
		Username:  gconv.String(doMember.Username),
		Ip:        ip,
		UserAgent: userAgent,
		Msg:       "登录成功",
		Module:    "微信登录",
	})

	key := gconv.String(doMember.Id) + "-" + gmd5.MustEncryptString(gconv.String(doMember.Username)) + gmd5.MustEncryptString(gconv.String(doMember.Password))
	if g.Cfg().MustGet(ctx, "gfToken.multiLogin").Bool() {
		key = gconv.String(doMember.Id) + "-" + gmd5.MustEncryptString(gconv.String(doMember.Username)) + gmd5.MustEncryptString(gconv.String(doMember.Password)+ip+userAgent)
	}
	memberInfo, err = service2.UCenterMembers().GetById(ctx, gconv.Uint64(doMember.Id))
	token, err = service2.GfToken().GenerateToken(ctx, key, memberInfo)
	if err != nil {
		g.Log().Error(ctx, err)
		err = gerror.New("登录失败，" + err.Error())
		return
	}
	r.Cookie.SetCookie("member_token", gurl.Encode(token), r.Request.Host, "/", 0)
	res = &wechat.WxLogin2Res{
		Token: token,
	}
	return
}
